<?php

class BWFAN_Wabot_WAGroupAction extends BWFAN_Action {

    private static $instance = null;

    public function __construct() {
        $this->action_name = __( 'Incluir/Remover Contato em grupo do WhatsApp', 'autonami-automations-connectors' );
        $this->action_desc = __( 'Adicionar ou remover um contato de um grupo do WhatsApp via Wabot', 'autonami-automations-connectors' );
        $this->support_v2  = true;
    }

    public static function get_instance() {
        if ( null === self::$instance ) {
            self::$instance = new self();
        }

        return self::$instance;
    }

    public function make_v2_data( $automation_data, $step_data ) {
        $data_to_set = array(
            'number'		=> BWFAN_Common::decode_merge_tags( $step_data['phone'] ?? '' ),
            'group_id' 		=> $step_data['group_id'] ?? '',
            'action' 		=> $step_data['action'] ?? '',
            'instance_id'	=> $step_data['connector_data']['instance_id'] ?? '',
            'access_token'	=> $step_data['connector_data']['access_token'] ?? '',
            'subdomain'		=> $step_data['connector_data']['subdomain'] ?? '',
        );

        return $data_to_set;
    }

	public function execute_action( $action_data ) {
		global $wpdb;
		$this->set_data( $action_data['processed_data'] );
		$this->data['task_id'] = $action_data['task_id'];
	
		/** Attaching track id */
		$sql_query         = 'Select meta_value FROM {table_name} WHERE bwfan_task_id = %d AND meta_key = %s';
		$sql_query         = $wpdb->prepare( $sql_query, $this->data['task_id'], 't_track_id' ); //phpcs:ignore WordPress.DB.PreparedSQL
		$gids              = BWFAN_Model_Taskmeta::get_results( $sql_query );
		$this->data['gid'] = '';
		if ( ! empty( $gids ) && is_array( $gids ) ) {
			foreach ( $gids as $gid ) {
				$this->data['gid'] = $gid['meta_value'];
			}
		}
	
		/** Validating connector */
		$load_connector = WFCO_Load_Connectors::get_instance();
		$call_class     = $load_connector->get_call( 'wfco_wabot_add_to_contactgroup' );
		if ( is_null( $call_class ) ) {
			$this->progress = false;
	
			return array(
				'status'  => 4,
				'message' => __( 'Adicionar ao grupo de contato chamada não encontrada', 'autonami-automations-connectors' ),
			);
		}
	
		$integration = BWFAN_Wabot_Integration::get_instance();
		$settings = $integration->get_settings();
		$this->data['access_token'] = $settings['access_token'];
		$this->data['instance_id']  = $settings['instance_id'];
		$this->data['subdomain']    = $settings['subdomain'];
	
		/** WC order case */
		if ( ! empty( $this->data['order_id'] ) ) {
			$order_details = wc_get_order( $this->data['order_id'] );
	
			/** Appending country code */
			$country = $order_details->get_billing_country();
			if ( ! empty( $country ) ) {
				$this->data['country_code'] = $country;
			}
		} elseif ( ! empty( $this->data['cart_abandoned_id'] ) ) {
			/** Cart abandonment case */
			$cart_details = BWFAN_Merge_Tag_Loader::get_data( 'cart_details' );
	
			/** Appending country code in case available */
			$checkout_data = json_decode( $cart_details['checkout_data'], true );
			if ( is_array( $checkout_data ) && isset( $checkout_data['fields'] ) && isset( $checkout_data['fields']['billing_country'] ) && ! empty( $checkout_data['fields']['billing_country'] ) ) {
				$this->data['country_code'] = $checkout_data['fields']['billing_country'];
			}
		}
	
		$call_class->set_data( $this->data );
		
		$response = $call_class->process();
		if ( is_array( $response ) && 200 === $response['response'] && ( isset( $response['body']['status'] ) && $response['body']['status'] === 'success' ) ) {
			$this->progress = false;
	
			return array(
				'status'  => 3,
				'message' => __( 'Contato adicionado ao grupo com sucesso.', 'autonami-automations-connectors' ),
			);
		}
	
		$message = __( 'Não foi possível adicionar o contato. ', 'autonami-automations-connectors' );
		$status  = 4;
	
		if ( isset( $response['body']['error'] ) && isset( $response['body']['error'] ) && isset( $response['body']['message'] ) ) {
			$message = $response['body']['message'];
		} elseif ( isset( $response['body']['message'] ) ) {
			$message = $response['body']['message'];
		} elseif ( isset( $response['bwfan_response'] ) && ! empty( $response['bwfan_response'] ) ) {
			$message = $response['bwfan_response'];
		} elseif ( is_array( $response['body'] ) && isset( $response['body'][0] ) && is_string( $response['body'][0] ) ) {
			$message = $message . $response['body'][0];
		}
		$this->progress = false;
	
		return array(
			'status'  => $status,
			'message' => $message,
		);
	}
	
	public function before_executing_task() {
		add_filter( 'bwfan_change_tasks_retry_limit', [ $this, 'modify_retry_limit' ], 99 );
	}

	public function after_executing_task() {
		remove_filter( 'bwfan_change_tasks_retry_limit', [ $this, 'modify_retry_limit' ], 99 );
	}

	public function modify_retry_limit( $retry_data ) {
		$retry_data[] = DAY_IN_SECONDS;

		return $retry_data;
	}

	public function handle_response_v2( $response ) {
		do_action( 'bwfan_sendsms_action_response', $response, $this->data );
		
		if ( is_array( $response ) && 200 === $response['response'] && ( isset( $response['body']['status'] ) && $response['body']['status'] === 'success' ) ) {
			$this->progress = false;
			return $this->success_message( __( 'Contato adicionado ao grupo com sucesso.', 'autonami-automations-connectors' ) );
		}

		$message = __( 'O contato não pôde ser adicionado ao grupo. ', 'autonami-automations-connectors' );
		
		if ( ( isset( $response['body']['status'] ) && $response['body']['status'] === 'error' )  && isset( $response['body']['message'] ) ){
			$message = $response['body']['message'];
		} elseif ( isset( $response['body']['message'] ) ) {
			$message = $response['body']['message'];
		} elseif ( isset( $response['bwfan_response'] ) && ! empty( $response['bwfan_response'] ) ) {
			$message = $response['bwfan_response'];
		} elseif ( is_array( $response['body'] ) && isset( $response['body'][0] ) && is_string( $response['body'][0] ) ) {
			$message = $message . $response['body'][0];
		}
		$this->progress = false;

		return $this->error_response( $message );
	}

	/**
	 * v2 Method: Get field Schema
	 *
	 * @return array[]
	 */
	public function get_fields_schema() {
		return [
			[
				'id'          => 'phone',
				'label'       => __( "Telefone de contato", 'wp-marketing-automations' ),
				'type'        => 'text',
				'placeholder' => "5551997181346",
				"class"       => 'bwfan-input-wrapper',
				'tip'         => __( '', 'autonami-automations-connectors' ),
				"description" => '',
				"required"    => true,
			],
			[
				'id'          => 'group_id',
				'label'       => __( "ID do grupo do WhatsApp", 'wp-marketing-automations' ),
				'type'        => 'text',
				'placeholder' => "XXXXXXXXXXXXXXXXX@g.us",
				"class"       => 'bwfan-input-wrapper',
				'tip'         => __( '', 'autonami-automations-connectors' ),
				"description" => '',
				"required"    => true,
			],
			[
				'id'          => 'action',
				'label'       => __( "Ação", 'wp-marketing-automations' ),
				'type'        => 'radio',
				'options'     => [
					[
						'label' => __( "Incluir", 'wp-marketing-automations' ),
						'value' => 'add',
					],
					[
						'label'   => __( "Remover", 'wp-marketing-automations' ),
						'value'   => 'remove',
					],
				],
				'tip'         => __( '', 'autonami-automations-connectors' ),
				"description" => '',
				"required"    => true,
			]
		];
	}
}	

return 'BWFAN_Wabot_WAGroupAction';
